# Copyright (c) 2019 Nico Alt
# SPDX-License-Identifier: AGPL-3.0-only
# License-Filename: LICENSE.md

from operator import itemgetter
from urllib.parse import urljoin

from requests import get as _get
from requests import post as _post

from briar_wrapper.constants import BASE_HTTP_URL
from briar_wrapper.model import Model


class Contacts(Model):

    API_ENDPOINT = "contacts/"
    CONNECTION_EVENTS = ("ContactConnectedEvent", "ContactDisconnectedEvent")

    _connections_callback = None

    def add_pending(self, link, alias):
        url = urljoin(BASE_HTTP_URL, self.API_ENDPOINT + "add/pending/")
        _post(url, headers=self._headers, json={"link": link, "alias": alias})

    def get(self):
        url = urljoin(BASE_HTTP_URL, self.API_ENDPOINT)
        request = _get(url, headers=self._headers)
        contacts = request.json()
        contacts = Contacts._sort_contact_list(contacts)
        return contacts

    def get_link(self):
        url = urljoin(BASE_HTTP_URL, self.API_ENDPOINT + "add/link/")
        request = _get(url, headers=self._headers).json()
        return request['link']

    def watch_connections(self, callback):
        self._connections_callback = callback
        signal_ids = list()
        event_callback = self._handle_connections_callback
        for event in self.CONNECTION_EVENTS:
            signal_id = self._api.socket_listener.connect(event,
                                                          event_callback)
            signal_ids.append(signal_id)
        return signal_ids

    def _handle_connections_callback(self, message):
        contact_id = message["data"]["contactId"]
        if message["name"] == "ContactConnectedEvent":
            self._connections_callback(contact_id, True)
        elif message["name"] == "ContactDisconnectedEvent":
            self._connections_callback(contact_id, False)
        else:
            raise Exception(f"Wrong event in callback: {message['name']}")

    def _sort_contact_list(contacts):
        contacts.sort(key=itemgetter("lastChatActivity"),
                      reverse=True)
        return contacts
