# Copyright (c) 2019 Nico Alt
# SPDX-License-Identifier: AGPL-3.0-only
# License-Filename: LICENSE.md

from urllib.parse import urljoin

import requests

from briar_wrapper.constants import BASE_HTTP_URL
from briar_wrapper.model import Model


class PrivateChat(Model):

    API_ENDPOINT = "messages/"

    def __init__(self, api, contact_id):
        super().__init__(api)
        self._contact_id = contact_id

    def get(self):
        url = urljoin(BASE_HTTP_URL,
                      self.API_ENDPOINT + str(self._contact_id))
        request = requests.get(url, headers=self._headers)
        return request.json()

    def send(self, message):
        url = urljoin(BASE_HTTP_URL,
                      self.API_ENDPOINT + str(self._contact_id))
        requests.post(url, headers=self._headers, json={"text": message})
